@@ -205,6 +205,10 @@ impl<'a> FnAnalyzer<'a> {
         };
         Api {
             name: api.name,
+            // An "extra API" is unlikely to have an `original_name, but we
+            // don't need to knoww that here -- we can simply pass on
+            // `api.original_name`.
+            original_name: api.original_name,
             deps: api.deps,
             detail: new_detail,
         }
@@ -221,7 +225,11 @@ impl<'a> FnAnalyzer<'a> {
             ApiDetail::ConcreteType { rs_definition } => ApiDetail::ConcreteType { rs_definition },
             ApiDetail::StringConstructor => ApiDetail::StringConstructor,
             ApiDetail::Function { fun, analysis: _ } => {
-                let analysis = self.analyze_foreign_fn(&api.name.get_namespace(), &fun)?;
+                let analysis = self.analyze_foreign_fn(
+                    &api.name.get_namespace(),
+                    &fun,
+                    api.original_name.clone(),
+                )?;
                 match analysis {
                     None => return Ok(None),
                     Some(FnAnalysisResult(analysis, id, fn_deps)) => {
@@ -247,6 +255,7 @@ impl<'a> FnAnalyzer<'a> {
         };
         Ok(Some(Api {
             name: QualifiedName::new(api.name.get_namespace(), new_id),
+            original_name: api.original_name,
             deps: new_deps,
             detail: api_detail,
         }))
@@ -315,6 +324,7 @@ impl<'a> FnAnalyzer<'a> {
         &mut self,
         ns: &Namespace,
         func_information: &FuncToConvert,
+        original_name: Option<String>,
     ) -> Result<Option<FnAnalysisResult>, ConvertErrorWithContext> {
         let fun = &func_information.item;
         let virtual_this = &func_information.virtual_this_type;
@@ -327,7 +337,6 @@ impl<'a> FnAnalyzer<'a> {
         if initial_rust_name.ends_with("_destructor") {
             return Ok(None);
         }
-        let original_name = Self::get_bindgen_original_name_annotation(&fun);
         let diagnostic_display_name = original_name.as_ref().unwrap_or(&initial_rust_name);
 
         // Now let's analyze all the parameters.
@@ -482,6 +491,13 @@ impl<'a> FnAnalyzer<'a> {
             return Err(contextualize_error(ConvertError::UnusedTemplateParam));
         }
 
+        // Reject move constructors.
+        if Self::is_move_constructor(&fun) {
+            return Err(contextualize_error(
+                ConvertError::MoveConstructorUnsupported,
+            ));
+        }
+
         // Analyze the return type, just as we previously did for the
         // parameters.
         let mut return_analysis = if let FnKind::Method(ref self_ty, MethodKind::Constructor) = kind
@@ -801,11 +817,11 @@ impl<'a> FnAnalyzer<'a> {
         Ok(result)
     }
 
-    fn get_bindgen_original_name_annotation(fun: &ForeignItemFn) -> Option<String> {
+    fn get_bindgen_special_member_annotation(fun: &ForeignItemFn) -> Option<String> {
         fun.attrs
             .iter()
             .filter_map(|a| {
-                if a.path.is_ident("bindgen_original_name") {
+                if a.path.is_ident("bindgen_special_member") {
                     let r: Result<LitStr, syn::Error> = a.parse_args();
                     match r {
                         Ok(ls) => Some(ls.value()),
@@ -818,6 +834,10 @@ impl<'a> FnAnalyzer<'a> {
             .next()
     }
 
+    fn is_move_constructor(fun: &ForeignItemFn) -> bool {
+        Self::get_bindgen_special_member_annotation(fun).map_or(false, |val| val == "move_ctor")
+    }
+
     fn get_reference_parameters_and_return(fun: &ForeignItemFn) -> (HashSet<Ident>, bool) {
         let mut ref_params = HashSet::new();
         let mut ref_return = false;
