@@ -27,7 +27,7 @@ use std::collections::HashMap;
 pub(crate) use non_pod_struct::make_non_pod;
 
 use proc_macro2::TokenStream;
-use syn::{parse_quote, ForeignItem, Ident, Item, ItemForeignMod, ItemMod};
+use syn::{parse_quote, Attribute, ForeignItem, Ident, Item, ItemForeignMod, ItemMod};
 
 use crate::types::{make_ident, Namespace, QualifiedName};
 use impl_item_creator::create_impl_items;
@@ -38,6 +38,9 @@ use self::{
     non_pod_struct::new_non_pod_struct,
 };
 
+use super::codegen_cpp::type_to_cpp::{
+    namespaced_name_using_original_name_map, original_name_map_from_apis, OriginalNameMap,
+};
 use super::{
     analysis::fun::FnAnalysis,
     api::{Api, ApiAnalysis, ApiDetail, ImplBlockDetails, TypeKind, TypedefKind},
@@ -115,6 +118,7 @@ fn remove_nones<T>(input: Vec<Option<T>>) -> Vec<T> {
 pub(crate) struct RsCodeGenerator<'a> {
     include_list: &'a [String],
     bindgen_mod: ItemMod,
+    original_name_map: OriginalNameMap,
 }
 
 impl<'a> RsCodeGenerator<'a> {
@@ -127,6 +131,7 @@ impl<'a> RsCodeGenerator<'a> {
         let c = Self {
             include_list,
             bindgen_mod,
+            original_name_map: original_name_map_from_apis(&all_apis),
         };
         c.rs_codegen(all_apis)
     }
@@ -143,7 +148,7 @@ impl<'a> RsCodeGenerator<'a> {
             .into_iter()
             .map(|api| {
                 let more_cpp_needed = api.additional_cpp().is_some();
-                let gen = Self::generate_rs_for_api(&api.name, api.detail);
+                let gen = self.generate_rs_for_api(&api.name, api.detail);
                 ((api.name, gen), more_cpp_needed)
             })
             .unzip();
@@ -308,6 +313,54 @@ impl<'a> RsCodeGenerator<'a> {
         }));
     }
 
+    fn remove_bindgen_attrs_from_item(mut item: Item) -> Item {
+        fn is_bindgen_attr(attr: &Attribute) -> bool {
+            let segments = &attr.path.segments;
+            segments.len() == 1
+                && segments
+                    .first()
+                    .unwrap()
+                    .ident
+                    .to_string()
+                    .starts_with("bindgen_")
+        }
+
+        fn remove_bindgen_attrs(attrs: &mut Vec<Attribute>) {
+            *attrs = attrs
+                .drain(..)
+                .filter(|attr| !is_bindgen_attr(&attr))
+                .collect();
+        }
+
+        match &mut item {
+            Item::Const(item_const) => remove_bindgen_attrs(&mut item_const.attrs),
+            Item::Enum(item_enum) => remove_bindgen_attrs(&mut item_enum.attrs),
+            Item::ExternCrate(item_extern_crate) => {
+                remove_bindgen_attrs(&mut item_extern_crate.attrs)
+            }
+            Item::Fn(item_fn) => remove_bindgen_attrs(&mut item_fn.attrs),
+            Item::ForeignMod(item_foreign_mod) => remove_bindgen_attrs(&mut item_foreign_mod.attrs),
+            Item::Impl(item_impl) => remove_bindgen_attrs(&mut item_impl.attrs),
+            Item::Macro(item_macro) => remove_bindgen_attrs(&mut item_macro.attrs),
+            Item::Macro2(item_macro2) => remove_bindgen_attrs(&mut item_macro2.attrs),
+            Item::Mod(item_mod) => remove_bindgen_attrs(&mut item_mod.attrs),
+            Item::Static(item_static) => remove_bindgen_attrs(&mut item_static.attrs),
+            Item::Struct(item_struct) => remove_bindgen_attrs(&mut item_struct.attrs),
+            Item::Trait(item_trait) => remove_bindgen_attrs(&mut item_trait.attrs),
+            Item::TraitAlias(item_trait_alias) => remove_bindgen_attrs(&mut item_trait_alias.attrs),
+            Item::Type(item_type) => remove_bindgen_attrs(&mut item_type.attrs),
+            Item::Union(item_union) => remove_bindgen_attrs(&mut item_union.attrs),
+            Item::Use(item_use) => remove_bindgen_attrs(&mut item_use.attrs),
+            Item::Verbatim(_) => {}
+            #[cfg(test)]
+            Item::__TestExhaustive(_) => unimplemented!(),
+            #[cfg(not(test))]
+            // Do nothing -- assumne there are no Attributes in the item.
+            _ => (),
+        }
+        item
+    }
+
     fn append_child_bindgen_namespace(
         &mut self,
         ns_entries: &NamespaceEntries<(QualifiedName, RsCodegenResult)>,
@@ -317,7 +370,15 @@ impl<'a> RsCodeGenerator<'a> {
     ) {
         let mut impl_entries_by_type: HashMap<_, Vec<_>> = HashMap::new();
         for item in ns_entries.entries() {
-            output_items.extend(item.1.bindgen_mod_item.iter().cloned());
+            // Remove `bindgen_` attributes. They don't have a corresponding macro defined anywhere,
+            // so they will cause compilation errors if we leave them in.
+            output_items.extend(
+                item.1
+                    .bindgen_mod_item
+                    .iter()
+                    .cloned()
+                    .map(|item| Self::remove_bindgen_attrs_from_item(item)),
+            );
             if let Some(impl_entry) = &item.1.impl_entry {
                 impl_entries_by_type
                     .entry(impl_entry.ty.clone())
@@ -374,6 +435,7 @@ impl<'a> RsCodeGenerator<'a> {
     }
 
     fn generate_rs_for_api(
+        &self,
         name: &QualifiedName,
         api_detail: ApiDetail<FnAnalysis>,
     ) -> RsCodegenResult {
@@ -390,17 +452,17 @@ impl<'a> RsCodeGenerator<'a> {
                 materialization: Use::Unused,
             },
             ApiDetail::ConcreteType { .. } => RsCodegenResult {
-                global_items: Self::generate_extern_type_impl(TypeKind::NonPod, &name),
+                global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
                 bridge_items: create_impl_items(&id),
-                extern_c_mod_item: Some(ForeignItem::Verbatim(Self::generate_cxxbridge_type(name))),
+                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
                 bindgen_mod_item: Some(Item::Struct(new_non_pod_struct(id.clone()))),
                 impl_entry: None,
                 materialization: Use::Unused,
             },
             ApiDetail::ForwardDeclaration => RsCodegenResult {
-                extern_c_mod_item: Some(ForeignItem::Verbatim(Self::generate_cxxbridge_type(name))),
+                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
                 bridge_items: Vec::new(),
-                global_items: Self::generate_extern_type_impl(TypeKind::NonPod, &name),
+                global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
                 bindgen_mod_item: Some(Item::Struct(new_non_pod_struct(id))),
                 impl_entry: None,
                 materialization: Use::UsedFromCxxBridge,
@@ -431,14 +493,14 @@ impl<'a> RsCodeGenerator<'a> {
                 bindgen_mod_item,
                 analysis,
             } => RsCodegenResult {
-                global_items: Self::generate_extern_type_impl(analysis, &name),
+                global_items: self.generate_extern_type_impl(analysis, &name),
                 impl_entry: None,
                 bridge_items: if analysis.can_be_instantiated() {
                     create_impl_items(&id)
                 } else {
                     Vec::new()
                 },
-                extern_c_mod_item: Some(ForeignItem::Verbatim(Self::generate_cxxbridge_type(name))),
+                extern_c_mod_item: Some(ForeignItem::Verbatim(self.generate_cxxbridge_type(name))),
                 bindgen_mod_item,
                 materialization: Use::UsedFromCxxBridge,
             },
@@ -513,8 +575,8 @@ impl<'a> RsCodeGenerator<'a> {
         })
     }
 
-    fn generate_extern_type_impl(type_kind: TypeKind, tyname: &QualifiedName) -> Vec<Item> {
-        let tynamestring = tyname.to_cpp_name();
+    fn generate_extern_type_impl(&self, type_kind: TypeKind, tyname: &QualifiedName) -> Vec<Item> {
+        let tynamestring = namespaced_name_using_original_name_map(tyname, &self.original_name_map);
         let fulltypath = tyname.get_bindgen_path_idents();
         let kind_item = match type_kind {
             TypeKind::Pod => "Trivial",
@@ -529,11 +591,19 @@ impl<'a> RsCodeGenerator<'a> {
         })]
     }
 
-    fn generate_cxxbridge_type(name: &QualifiedName) -> TokenStream {
-        let id = name.get_final_ident();
+    fn generate_cxxbridge_type(&self, name: &QualifiedName) -> TokenStream {
         let ns = name.get_namespace();
-        let mut for_extern_c_ts = if !ns.is_empty() {
-            let ns_string = ns.iter().cloned().collect::<Vec<String>>().join("::");
+        let id = name.get_final_ident();
+        let mut ns_components: Vec<String> = ns.iter().cloned().collect();
+        let mut cxx_name = None;
+        if let Some(original_name) = self.original_name_map.get(name) {
+            let mut name_components: Vec<_> = original_name.split("::").collect();
+            cxx_name = Some(name_components.pop().unwrap());
+            ns_components.extend(name_components.iter().map(|s| s.to_string()));
+        };
+
+        let mut for_extern_c_ts = if !ns_components.is_empty() {
+            let ns_string = ns_components.join("::");
             quote! {
                 #[namespace = #ns_string]
             }
@@ -541,6 +611,12 @@ impl<'a> RsCodeGenerator<'a> {
             TokenStream::new()
         };
 
+        if let Some(n) = cxx_name {
+            for_extern_c_ts.extend(quote! {
+                #[cxx_name = #n]
+            });
+        }
+
         for_extern_c_ts.extend(quote! {
             type #id = super::bindgen::root::
         });
