@@ -45,6 +45,9 @@ pub enum ConvertError {
     Blocked(QualifiedName),
     UnusedTemplateParam,
     TooManyUnderscores,
+    UnknownDependentType,
+    IgnoredDependent,
+    MoveConstructorUnsupported,
 }
 
 fn format_maybe_identifier(id: &Option<Ident>) -> String {
@@ -82,6 +85,9 @@ impl Display for ConvertError {
             ConvertError::Blocked(tn) => write!(f, "Found an attempt at using a type marked as blocked! ({})", tn.to_cpp_name())?,
             ConvertError::UnusedTemplateParam => write!(f, "This function or method uses a type where one of the template parameters was incomprehensible to bindgen/autocxx - probably because it uses template specialization.")?,
             ConvertError::TooManyUnderscores => write!(f, "Names containing __ are reserved by C++ so not acceptable to cxx")?,
+            ConvertError::UnknownDependentType => write!(f, "This item relies on a type not known to autocxx.")?,
+            ConvertError::IgnoredDependent => write!(f, "This item depends on some other type which autocxx could not generate.")?,
+            ConvertError::MoveConstructorUnsupported => write!(f, "This is a move constructor, for which we currently cannot generate bindings.")?,
         }
         Ok(())
     }
